﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ALPS {
  [Item("MatingPoolCreator", "An operator which creating mating pools based on a set of sub-populations. For each sub-population, the individuals from the previous sub-population are copied into the current sub-population.")]
  [StorableClass]
  public sealed class MatingPoolCreator : SingleSuccessorOperator {

    [StorableConstructor]
    private MatingPoolCreator(bool deserializing)
      : base(deserializing) { }
    private MatingPoolCreator(MatingPoolCreator original, Cloner cloner)
      : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MatingPoolCreator(this, cloner);
    }
    public MatingPoolCreator()
      : base() {
    }

    public override IOperation Apply() {
      var subPopulations = ExecutionContext.Scope.SubScopes;

      for (int i = subPopulations.Count - 1; i >= 0; i++) {
        var currentSubPopulation = subPopulations[i].SubScopes;
        var prevSubPopulation = i > 0 ? subPopulations[i - 1].SubScopes : Enumerable.Empty<IScope>();

        foreach (var individual in prevSubPopulation) {
          currentSubPopulation.Add(individual);
        }
      }
      return base.Apply();
    }
  }
}